"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.baseQueryReauth = exports.baseQuery = void 0;
const async_mutex_1 = require("async-mutex");
const slice_1 = require("@procyon/auth/slice");
const utils_1 = require("@procyon/utils");
const mutex = new async_mutex_1.Mutex();
const baseQuery = ({ baseUrl, authentication }) => ({ url, method = 'GET', params, body, headers }, { getState }) => __awaiter(void 0, void 0, void 0, function* () {
    if (authentication && getState().procyon.auth !== undefined) {
        const state = getState();
        const { authenticated, accessToken } = (0, slice_1.getAuthAttributes)(state);
        if (authenticated) {
            headers = Object.assign(Object.assign({}, headers), { [authentication.headerName]: `${authentication.tokenType} ${accessToken === null || accessToken === void 0 ? void 0 : accessToken.toString()}`.trim() });
        }
    }
    try {
        const fetchUrl = new URL(`${baseUrl}${url}`);
        const fetchParams = new URLSearchParams(fetchUrl.search);
        if (params != undefined) {
            for (const [key, value] of Object.entries(params)) {
                fetchParams.append(key, value);
            }
        }
        const response = yield fetch(`${fetchUrl.href}?${fetchParams.toString()}`, Object.assign({ headers: Object.assign({ 'Content-Type': 'application/json' }, headers), method: method.toUpperCase() }, (body ? { body: JSON.stringify(body) } : {})));
        const json = yield response.text().then((text) => ((0, utils_1.isJSON)(text) ? JSON.parse(text) : text));
        if (!response.ok)
            throw response;
        return { data: json };
    }
    catch (err) {
        return { error: { status: err.status, error: err } };
    }
});
exports.baseQuery = baseQuery;
const baseQueryReauth = ({ baseUrl, authentication, when401, }) => (args, api, extraOptions) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    yield mutex.waitForUnlock();
    let result = yield (0, exports.baseQuery)({ baseUrl, authentication })(args, api, extraOptions);
    if (((_a = result.error) === null || _a === void 0 ? void 0 : _a.status) === 401) {
        if (!mutex.isLocked()) {
            const release = yield mutex.acquire();
            try {
                yield when401(api);
            }
            finally {
                release();
            }
        }
        else {
            yield mutex.waitForUnlock();
            result = yield (0, exports.baseQuery)({ baseUrl, authentication })(args, api, extraOptions);
        }
    }
    return result;
});
exports.baseQueryReauth = baseQueryReauth;
